<?php

namespace EventPortal\Domain\Approval\Models;

use EventPortal\Concerns\AddToFillable;
use EventPortal\Concerns\HasStatus;
use EventPortal\Contracts\Statusable;
use EventPortal\Domain\Comment\Models\Comment;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Util\Status;
use Iben\Statable\Statable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\MorphMany;
use Illuminate\Database\Eloquent\Relations\Pivot;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;

class Approval extends Pivot implements AuditableInterface, Statusable
{
    use Statable;
    use HasStatus;
    use Auditable;
    use Statable;
    use AddToFillable;

    const KEY = 'eventApproval';

    /** @var string */
    protected $table = 'approvals';

    /** @var array<string> */
    protected $fillable = [
        'event_id',
        'admin_id',
        'status',
    ];

    public $incrementing = true;

    public function comments(): MorphMany
    {
        return $this->morphMany(Comment::class, 'commentable');
    }

    public function event(): BelongsTo
    {
        return $this->belongsTo(Event::class);
    }

    public function data(): HasMany
    {
        return $this->hasMany(ApprovalData::class, 'approval_id');
    }

    public function transitionRouteName(): string
    {
        return 'admin.event-approval.transition';
    }

    public function transitionRouteParameters(): array
    {
        return [
            'event' => $this->id,
        ];
    }

    protected function getGraph(): string
    {
        $event = $this->event()->first();

        switch ($event->approval_type) {
            case 'novoVisionMiff':
            default:
                return 'novoVisionMiff';
        }

        return 'novoVisionMiff';
    }

    public function approvedAndRelevant(): bool
    {
        if ($this->status !== 'approved') {
            return false;
        }

        return now()->subDays(3)->gte($this->updated_at->format('Y-m-d'));
    }

    public function doesCurrentStatusMatchNewStatus(string $status): bool
    {
        return $this->status === $status;
    }

    public function scopeActionableByLineManager(Builder $query): Builder
    {
        return $query->whereIn('approvals.status', ['awaiting-approval']);
    }

    public function scopeActionableByCompliance(Builder $query): Builder
    {
        return $query->whereIn('approvals.status', ['line-manager-approved']);
    }

    public function getStatusTitleAttribute(): string
    {
        return Status::make($this->getGraph())->getReadable($this->attributes['status']);
    }

    public function getStatusDescriptionAttribute(string $title): string
    {
        return trans(Status::make($this->getGraph())->getDescription($this->attributes['status']), ['event' => $title]);
    }

    public function getStatusAlertAttribute(): ?string
    {
        return Status::make($this->getGraph())->getAlert($this->attributes['status']);
    }
}
